* PRISMOD MODEL - dopris3_modelavg.do *
* Libor Dusek, Jan Vavra, Paolo Buonanno
** more info in dopris0_readme.do


* contains program pminflowsimul with argument p, representing name of scenario, that simulates prison population 

global pcent "100" 
quietly {
* Define program for generation of alternative sentences stock at risk and inflow from them, per oca
capture program drop simul_stockatrisk
	program define simul_stockatrisk
		args scen_name inflow_n outflow_rate trialper_length am_var end_suffix		
	quietly {
	replace `trialper_length'=0 if missing(`trialper_length')
	replace `inflow_n'=0 if missing(`inflow_n')
	replace `outflow_rate' =0 if missing(`outflow_rate' )
	replace `am_var'=0 if missing(`am_var')	
	* 1. Generate inflow matrix
		forval tc = 128 (1) $end_horizon {			
			by oca: egen cohr_inflow_`tc'=sum(`inflow_n') if q==`tc'
			replace cohr_inflow_`tc'=0 if q!=`tc'		
			g cohr_inflowF_`tc'=cohr_inflow_`tc'
			replace cohr_inflowF_`tc'=cohr_inflowF_`tc'[_n-1] if oca==oca[_n-1] & q>`tc'

		}
	* 2. Generate lambda
		forvalues oca = 1 (1) 9 {
				forvalues tc = 128 (1) $end_horizon {					
					sum  `trialper_length' if q==`tc' & oca==`oca'
					scalar parole_l=r(mean)						
					capture g cohr_lambda_`tc'=0
					replace cohr_lambda_`tc'=1 if q>=`tc' & q<`tc'+int(parole_l) & oca==`oca'					
					replace cohr_lambda_`tc'=parole_l-int(parole_l) if q==`tc'+int(parole_l) & oca==`oca' & int(parole_l)!=parole_l
					replace cohr_lambda_`tc'=0 if  cohr_lambda_`tc'<0
				}
		}	
		forval tc = 128 (1) $end_horizon {	
			replace cohr_inflow_`tc'=0 if q==`tc' & cohr_lambda_`tc'==0
		}		
		forval tc = 128 (1) $end_horizon {	
			g cohr_lambdaF_`tc'=cohr_lambda_`tc'
			replace cohr_lambdaF_`tc'=1 if (cohr_lambdaF_`tc'<1 & cohr_lambdaF_`tc'>0)	
		}
			
	* 3. Generate coefficients	
		g cohr_exitrate=`outflow_rate' + F.`am_var'
		forval tc = 128 (1) $end_horizon {			
			g cohr_coeff_`tc'=cohr_lambdaF_`tc'*L.cohr_exitrate if q>128			
			replace cohr_coeff_`tc'=1 if `tc'==q & cohr_lambda_`tc'!=0
			* chain exitrates and multiply by lambda
			replace cohr_coeff_`tc'=((1-cohr_coeff_`tc')*cohr_coeff_`tc'[_n-1])*cohr_lambda_`tc' if `tc'<q & oca==oca[_n-1] &  q>128
		}
	* 4. Generate stock and outflows
		forval tc = 128 (1) $end_horizon {
			g cohr_stock_`tc'=cohr_coeff_`tc'*cohr_inflowF_`tc'
		}
		* generate netflow
		forval tc = 128 (1) $end_horizon {
			g cohr_netflow_`tc'=cohr_stock_`tc'-cohr_stock_`tc'[_n-1] if q>128 & oca==oca[_n-1]
		}
		* generate outflow
		** substract inflow from netflow
		forval tc = 128 (1) $end_horizon {
			g cohr_temp_`tc'=cohr_netflow_`tc'-cohr_inflow_`tc'
			replace cohr_temp_`tc'=abs(cohr_temp_`tc')
			g cohr_outflow_`tc'= cohr_temp_`tc'[_n+1] if q>128 & oca==oca[_n-1]
		}
	* 5. Generate summary variables and finish		
		* stock per oca
		egen potastock_`end_suffix'=rowtotal(cohr_stock_*)
		egen potainflow_`end_suffix'=rowtotal(cohr_inflow_*)
		egen potaoutflow_`end_suffix'=rowtotal(cohr_outflow_*)
		
		* totals
		egen totpotastock_`end_suffix'=total(potastock_`end_suffix'), by(q)
		egen totpotainflow_`end_suffix'=total(potainflow_`end_suffix'), by(q)
		egen totpotaoutflow_`end_suffix'=total(potaoutflow_`end_suffix'), by(q)
		
		* drop technical matrices
		drop cohr_inflowF*
		drop cohr_lambdaF*		
		
		* generate inflows to prison from potential stocks
		capture drop inflowr_`end_suffix'
		g inflowr_`end_suffix'=potastock_`end_suffix'*`outflow_rate'
		* replace vra with zero of missing 
		replace inflowr_`end_suffix'=0 if missing(inflowr_`end_suffix')	
		
		* drop cohort variables
		drop cohr*		
	}	
end
	
*** SIMULATING PRISONERS STOCKS
capture program  drop  simul_prisonstock
program define simul_prisonstock
	args scen_name inflowa sentence prob_parole prob_nextsentence custody amnesty_var end
	
	quietly {
	* make sure that used wars are not missing
	replace `inflowa'=0 if missing(`inflowa')
	replace  `sentence'=0 if missing( `sentence')
	replace `prob_parole'=0 if missing(`prob_parole')
	replace `prob_nextsentence'=0 if missing(`prob_nextsentence')
	replace `custody'=0 if missing(`custody')
	replace `amnesty_var'=0 if missing(`amnesty_var')

	*** SPECIAL TREATMENT FOR RP PRISONERS
	**** SPLIT FOR HS, LSHS and LS prisoners
	if "`end'"=="rp" {
		** HS PRISONERS STOCK
		* 1. INFLOW matrix
		forvalues tc = 128 (1) $end_horizon {	
			by oca: egen cohr_inflow_`tc'=sum(`inflowa'*p4_hs) if q==`tc'
			replace cohr_inflow_`tc'=0 if q!=`tc' 			
		}
		* 2. LAMBDA, time dimension, 1 if cohort is present, fraction in last quarter, zero othervise
		** sentence is lowered by average custody
		g cohr_time_served=s_hs-`custody'
		replace cohr_time_served=0 if missing(cohr_time_served) | cohr_time_served<0
		forvalues oca = 1 (1) 9 {
				forvalues tc = 128 (1) $end_horizon {
					* make scalar for sentence lenght
					sum  cohr_time_served if q==`tc' & oca==`oca'
					scalar net_sentence=r(mean)	
					* generate cohort variable
					capture g cohr_lambda_`tc'=0
					* replace by 1 in whole quarters
					replace cohr_lambda_`tc'=1 if q>=`tc' & q<`tc'+int(net_sentence) & oca==`oca'
					* replace as fraction in partial quarter, make sure that int(parole_l)!=parole_l
					replace cohr_lambda_`tc'=net_sentence-int(net_sentence) if q==`tc'+int(net_sentence) & oca==`oca' & int(net_sentence)!=net_sentence
					replace cohr_lambda_`tc'=0 if  cohr_lambda_`tc'<0
					
					* 2.1 make eligible matrix, 1 in quarter where cohort is eligible to release to LS prison, 0 otherwise
					** fr_eligible defined on top of the file
					capture g cohr_eligible_`tc'=0
					if fr_eligible==0 {
					** if zero, same as lambda
						replace cohr_eligible_`tc'=cohr_lambda_`tc'
						replace cohr_eligible_`tc'=1 if cohr_eligible_`tc'>0 
					}
					if fr_eligible==1 {
					** cohr_eligible stays empty
					}
					else {
					replace cohr_eligible_`tc'=1 if q>=`tc'+round(net_sentence*fr_eligible) & q<=`tc'+int(net_sentence) & oca==`oca'				
					}
				}
		}
		
		* 3. Prepare variables
		** generate lagged lambda with exit coefficients
		forvalues tc = 128 (1) $end_horizon {
			g cohr_expired_`tc' =cohr_lambda_`tc'
			replace cohr_expired_`tc' =cohr_lambda_`tc'-cohr_lambda_`tc'[_n+1]  if  cohr_lambda_`tc'[_n+1]<1 &  cohr_lambda_`tc'[_n+1]>0  & oca==oca[_n-1]
		}
		* generate help lambda
		forvalues tc = 128 (1) $end_horizon {
			g cohr_help_`tc' =cohr_lambda_`tc'
			replace cohr_help_`tc' =. if  cohr_lambda_`tc'==0
		}		
		* generate other used vars
		forvalues tc = 128 (1) $end_horizon {
			g cohr_tempstock_`tc' =0
		}
		forvalues tc = 128 (1) $end_horizon {
			g cohr_eligiblestock_`tc' =0
		}
		forvalues tc = 128 (1) $end_horizon {
			g cohr_outflow_`tc' =0
		}
		forvalues tc = 128 (1) $end_horizon {
			g transfer_`tc' =0	
		}
		* 4. Generate outflows	
		forvalues row = 128 (1) $end_horizon {
			* generate value of temporary stock from beggining to the row
			forvalues tc = 128 (1) $end_horizon {		
					replace cohr_tempstock_`tc'=cohr_tempstock_`tc'[_n-1]+cohr_inflow_`tc'-cohr_outflow_`tc'[_n-1] if q<=`row' & oca==oca[_n-1] & q>128	
					replace cohr_tempstock_`tc'=round(cohr_tempstock_`tc', 0.00001)
					* generate eligible stock at temp stock * eligible matrix
					replace cohr_eligiblestock_`tc'=cohr_tempstock_`tc'*cohr_eligible_`tc'					
			}
			* generate sum of total stock vector
			egen sum_total=rowtotal(cohr_tempstock_*)
			replace sum_total=0 if missing(sum_total)
			* generate sum of eligible stock vector
			egen sum_eligible=rowtotal(cohr_eligiblestock_*) 
			replace sum_eligible=0 if missing(sum_eligible)
			* generate adjusted fraction of released vector
			g p_change_new=(sum_total/sum_eligible)*pe_hsls
			replace p_change_new=0 if missing(p_change_new)
			************************
			* check for possible problem, pe_o+outflow to ls can be bigger then 1		
				g check=0
				replace check=1 if (p_change_new+corr_hs*`prob_nextsentence')>1
				replace p_change_new=1-corr_hs*`prob_nextsentence' if check==1
			************************
			forvalues tc = 128 (1) $end_horizon {
					*** outflow to LS stock
					replace cohr_outflow_`tc'=cohr_outflow_`tc'+(p_change_new*cohr_eligiblestock_`tc') if q==`row' & oca==oca[_n-1]  & q>128
					*** transfer_tc is later used as inflow to lshs stock
					replace transfer_`tc'=cohr_outflow_`tc' if q==`row' & oca==oca[_n-1]  & q>128		
					*** outflow to next sentence
					replace cohr_outflow_`tc'=cohr_outflow_`tc'+(corr_hs*`prob_nextsentence'*cohr_tempstock_`tc') if q==`row' & oca==oca[_n-1]  & q>128
					* expired outflow
					replace cohr_outflow_`tc'=cohr_outflow_`tc'+((1-corr_hs*`prob_nextsentence'-p_change_new)*cohr_tempstock_`tc')*(cohr_expired_`tc') if q==`row' & oca==oca[_n-1]  & q>128 & cohr_expired_`tc'!=1 &  cohr_expired_`tc'!=0
					* get absolute outlow of last quarter (all remainign prisoners are released)
					replace cohr_outflow_`tc'=cohr_tempstock_`tc' if q==`row' & oca==oca[_n-1]  & q>128 & ( ( cohr_lambda_`tc'[_n+1]==0 & cohr_lambda_`tc'!=0 & cohr_lambda_`tc'[_n-1]!=0 ) | (cohr_lambda_`tc'[_n+1]==0 & cohr_lambda_`tc'!=0 & cohr_lambda_`tc'[_n-1]==0) )
					replace cohr_outflow_`tc'=0 if q==`row' & oca==oca[_n-1] & cohr_lambda_`tc'==0 	
					replace cohr_outflow_`tc'=round(cohr_outflow_`tc', 0.00001)
				}
			drop sum_*
			drop check
			drop p_change_new
		}
		* 3. Calculate stock and finalize
		forvalues tc = 128 (1) $end_horizon {
			g cohr_stock_`tc' =0
		}
		forvalues tc = 128 (1) $end_horizon {
			replace cohr_stock_`tc'=cohr_stock_`tc'[_n-1]+cohr_inflow_`tc'-cohr_outflow_`tc'[_n-1] if oca==oca[_n-1] & q>128	
		}
		*5. Create summary and needed variables	
		* total stock per oca
		egen stock_hs=rowtotal(cohr_stock*)
		egen inflow_hs=rowtotal(cohr_inflow*)
		egen outflow_hs=rowtotal(cohr_outflow*)
		egen totstock_hs= total(stock_hs), by(q)
		egen totinflow_hs = total(inflow_hs), by(q)
		egen totoutflow_hs = total(outflow_hs), by(q)
		
	
		noisily display "Regular HS done!" as result

		* drop matrices	
		drop cohr_stock*
		drop cohr_tempstock*
		drop cohr_outflow*
		drop cohr_inflow*

		** LSHS prisoners
		* 1. INFLOW matrix from hs stock
		forvalues tc = 128 (1) $end_horizon {	
			rename transfer_`tc' cohr_inflow_`tc'
		}	
		
		* 2. Generate outflow 
		** prepare variables
		forvalues tc = 128 (1) $end_horizon {
			g cohr_tempstock_`tc' =0
		}
		forvalues tc = 128 (1) $end_horizon {
			g cohr_outflow_`tc' =0
		}
		
		g cohr_exit_rate=`prob_nextsentence' + `prob_parole' + F.amv_rp
		forvalues row = 128 (1) $end_horizon {
			* generate value of stock
			forvalues tc = 128 (1) $end_horizon {		
					replace cohr_tempstock_`tc'=cohr_tempstock_`tc'[_n-1]+cohr_inflow_`tc'-cohr_outflow_`tc'[_n-1] if q<=`row' & oca==oca[_n-1] & q>128	
					replace cohr_tempstock_`tc'=round(cohr_tempstock_`tc', 0.00001)		
			}
			forvalues tc = 128 (1) $end_horizon {
					* get absolute outflow from precentage
					replace cohr_outflow_`tc'=cohr_outflow_`tc'+(cohr_exit_rate*cohr_tempstock_`tc') if q==`row' & oca==oca[_n-1]  & q>128
					* expired outflow
					replace cohr_outflow_`tc'=cohr_outflow_`tc'+((1-cohr_exit_rate)*cohr_tempstock_`tc')*(cohr_expired_`tc') if q==`row' & oca==oca[_n-1]  & q>128 & cohr_expired_`tc'!=1 &  cohr_expired_`tc'!=0
					* get absolute outlow of last quarter (all remainign prisoners are released)
					replace cohr_outflow_`tc'=cohr_tempstock_`tc' if q==`row' & oca==oca[_n-1]  & q>128 & ( ( cohr_lambda_`tc'[_n+1]==0 & cohr_lambda_`tc'!=0 & cohr_lambda_`tc'[_n-1]!=0 ) | (cohr_lambda_`tc'[_n+1]==0 & cohr_lambda_`tc'!=0 & cohr_lambda_`tc'[_n-1]==0) )
					replace cohr_outflow_`tc'=0 if q==`row' & oca==oca[_n-1] & cohr_lambda_`tc'==0 	
					replace cohr_outflow_`tc'=round(cohr_outflow_`tc', 0.00001)
			}
		}	
			
		* 3. Calculate stock and finalize
		forvalues tc = 128 (1) $end_horizon {
			g cohr_stock_`tc' =0
		}
		forvalues tc = 128 (1) $end_horizon {
			replace cohr_stock_`tc'=cohr_stock_`tc'[_n-1]+cohr_inflow_`tc'-cohr_outflow_`tc'[_n-1] if oca==oca[_n-1] & q>128	
			replace cohr_stock_`tc'=round(cohr_stock_`tc', 0.00001)	
		}
		* 4. Create summary and needed variables	
		* total stock per oca
		egen stock_lshs=rowtotal(cohr_stock*)
		egen inflow_lshs=rowtotal(cohr_inflow*)
		egen outflow_lshs=rowtotal(cohr_outflow*)
		egen totstock_lshs= total(stock_lshs), by(q)
		egen totinflow_lshs = total(inflow_lshs), by(q)
		egen totoutflow_lshs = total(outflow_lshs), by(q)		
		
		noisily display "Regular LSHS done!" as result

		drop cohr_*
		
		** LS PRISONERS STOCK, fast generation
		* 1. Generate inflow matrix
			forval tc = 128 (1) $end_horizon {	
				* inflows on diagonal
				by oca: egen cohr_inflow_`tc'=sum(`inflowa'*(1-p4_hs)) if q==`tc'
				replace cohr_inflow_`tc'=0 if q!=`tc'
				* inflowF is technical inflow matrix for computation, vector is filled by cohort inflow in all quarters
				g cohr_inflowF_`tc'=cohr_inflow_`tc'
				replace cohr_inflowF_`tc'=cohr_inflowF_`tc'[_n-1] if oca==oca[_n-1] & q>`tc'

			}
		* 2. Generate lambda
			** time served is lowered by custody
			g cohr_time_served=s_ls-`custody'
			** replace with zero if negative
			replace cohr_time_served=0 if missing(cohr_time_served) | cohr_time_served<0
			forvalues oca = 1 (1) 9 {
					forvalues tc = 128 (1) $end_horizon {
						* make scalar for parole length
						sum  cohr_time_served if q==`tc' & oca==`oca'
						scalar net_sentence=r(mean)	
						* generate cohort variable
						capture g cohr_lambda_`tc'=0
						* replace in whole quarters
						replace cohr_lambda_`tc'=1 if q>=`tc' & q<`tc'+int(net_sentence) & oca==`oca'
						* replace in partial quarter, make sure that int(parole_l)!=parole_l
						replace cohr_lambda_`tc'=net_sentence-int(net_sentence) if q==`tc'+int(net_sentence) & oca==`oca' & int(net_sentence)!=net_sentence
						replace cohr_lambda_`tc'=0 if  cohr_lambda_`tc'<0
					}
			}		
			forval tc = 128 (1) $end_horizon {	
				replace cohr_inflow_`tc'=0 if q==`tc' & cohr_lambda_`tc'==0
			}
			* generate lambdaF
			** technical lambda used for computation, last value is one, no partial quarters
			forval tc = 128 (1) $end_horizon {	
				g cohr_lambdaF_`tc'=cohr_lambda_`tc'
				replace cohr_lambdaF_`tc'=1 if (cohr_lambdaF_`tc'<1 & cohr_lambdaF_`tc'>0)	
			}
		* 3. Generate coefficients matrix
			** calculate total fraction of stock exited in quarter
			g cohr_exitrate= `prob_parole'+corr_ls*`prob_nextsentence'+F.`amnesty_var'
			forval tc = 128 (1) $end_horizon {
				* add exitrates when cohort is present
				g cohr_coeff_`tc'=cohr_lambdaF_`tc'*L.cohr_exitrate if q>128
				* replace value in first quarter with one (diagonal of matrix)
				replace cohr_coeff_`tc'=1 if `tc'==q & cohr_lambda_`tc'!=0
				* chain exitrates and multiply by lambda
				replace cohr_coeff_`tc'=((1-cohr_coeff_`tc')*cohr_coeff_`tc'[_n-1])*cohr_lambda_`tc' if `tc'<q & oca==oca[_n-1] &  q>128
			}
		* 4. Generate stock and outflows
			forval tc = 128 (1) $end_horizon {
				g cohr_stock_`tc'=cohr_coeff_`tc'*cohr_inflowF_`tc'
			}
			* generate netflow
			forval tc = 128 (1) $end_horizon {
				g cohr_netflow_`tc'=cohr_stock_`tc'-cohr_stock_`tc'[_n-1] if q>128 & oca==oca[_n-1]
			}
			* generate outflow
			** substract inflow from netflow
			forval tc = 128 (1) $end_horizon {
				g cohr_temp_`tc'=cohr_netflow_`tc'-cohr_inflow_`tc'
				replace cohr_temp_`tc'=abs(cohr_temp_`tc')
				g cohr_outflow_`tc'= cohr_temp_`tc'[_n+1] if q>128 & oca==oca[_n-1]
			}
		* 5. Generate summary variables and finish			
			* stock per oca
			egen stock_rpls=rowtotal(cohr_stock_*)
			egen inflow_rpls=rowtotal(cohr_inflow_*)
			egen outflow_rpls=rowtotal(cohr_outflow_*)
			* totals
			egen totstock_rpls=total(stock_rpls), by(q)
			egen totinflow_rpls=total(inflow_rpls), by(q)
			egen totoutflow_rpls=total(outflow_rpls), by(q)
			* drop technical matrices
			drop cohr_inflowF*
			drop cohr_lambdaF*			
			* drop cohor variables
			drop cohr*	
			
		noisily display "Regular LS done!" as result	
	}
	* else for alternative sentences prisoners
	else {
		* 1. Generate inflow matrix
			forval tc = 128 (1) $end_horizon {	
				* inflows on diagonal
				by oca: egen cohr_inflow_`tc'=sum(`inflowa') if q==`tc'
				replace cohr_inflow_`tc'=0 if q!=`tc'
				* inflowF is technical inflow matrix for computation, vector is filled by cohort inflow
				g cohr_inflowF_`tc'=cohr_inflow_`tc'
				replace cohr_inflowF_`tc'=cohr_inflowF_`tc'[_n-1] if oca==oca[_n-1] & q>`tc'
			}
		* 2. Generate lambda
			** time served is lowered by custody
			g cohr_time_served=`sentence'-`custody'
			** replace with zero if negative
			replace cohr_time_served=0 if missing(cohr_time_served) | cohr_time_served<0
			forvalues oca = 1 (1) 9 {
					forvalues tc = 128 (1) $end_horizon {
						* make scalar for parole length
						sum  cohr_time_served if q==`tc' & oca==`oca'
						scalar net_sentence=r(mean)	
						* generate cohort variable
						capture g cohr_lambda_`tc'=0
						* replace in whole quarters
						replace cohr_lambda_`tc'=1 if q>=`tc' & q<`tc'+int(net_sentence) & oca==`oca'
						* replace in partial quarter, make sure that int(parole_l)!=parole_l
						replace cohr_lambda_`tc'=net_sentence-int(net_sentence) if q==`tc'+int(net_sentence) & oca==`oca' & int(net_sentence)!=net_sentence
						replace cohr_lambda_`tc'=0 if  cohr_lambda_`tc'<0
					}
			}		
			forval tc = 128 (1) $end_horizon {	
				replace cohr_inflow_`tc'=0 if q==`tc' & cohr_lambda_`tc'==0
			}
			* generate lambdaF
			** technical lambda used for computation, last value is one, no partial quarters
			forval tc = 128 (1) $end_horizon {	
				g cohr_lambdaF_`tc'=cohr_lambda_`tc'
				replace cohr_lambdaF_`tc'=1 if (cohr_lambdaF_`tc'<1 & cohr_lambdaF_`tc'>0)	
			}			
		* 3. Generate coefficients
			** calculate total fraction of stock exited in quarter
			g cohr_exitrate=`prob_parole' +`prob_nextsentence'+F.`amnesty_var'
			forval tc = 128 (1) $end_horizon {
				* add exitrates when cohort is present
				g cohr_coeff_`tc'=cohr_lambdaF_`tc'*L.cohr_exitrate if q>128
				* replace value in first quarter with one (diagonal of matrix)
				replace cohr_coeff_`tc'=1 if `tc'==q & cohr_lambda_`tc'!=0
				* chain exitrates and multiply by lambda
				replace cohr_coeff_`tc'=((1-cohr_coeff_`tc')*cohr_coeff_`tc'[_n-1])*cohr_lambda_`tc' if `tc'<q & oca==oca[_n-1] &  q>128
			}
		* 4. Generate stock and outflows
			forval tc = 128 (1) $end_horizon {
				g cohr_stock_`tc'=cohr_coeff_`tc'*cohr_inflowF_`tc'
			}
			* generate netflow
			forval tc = 128 (1) $end_horizon {
				g cohr_netflow_`tc'=cohr_stock_`tc'-cohr_stock_`tc'[_n-1] if q>128 & oca==oca[_n-1]
			}
			* generate outflow
			** substract inflow from netflow
			forval tc = 128 (1) $end_horizon {
				g cohr_temp_`tc'=cohr_netflow_`tc'-cohr_inflow_`tc'
				replace cohr_temp_`tc'=abs(cohr_temp_`tc')
				g cohr_outflow_`tc'= cohr_temp_`tc'[_n+1] if q>128 & oca==oca[_n-1]
			}
		* 5. Generate summary variables and finish
			* stock per oca
			egen stock_`end'=rowtotal(cohr_stock_*)
			egen inflow_`end'=rowtotal(cohr_inflow_*)
			egen outflow_`end'=rowtotal(cohr_outflow_*)
			* totals
			egen totstock_`end'=total(stock_`end'), by(q)
			egen totinflow_`end'=total(inflow_`end'), by(q)
			egen totoutflow_`end'=total(outflow_`end'), by(q)
			* drop technical matrices
			drop cohr_inflowF*
			drop cohr_lambdaF*				
			* drop cohor variables
			drop cohr*	
	
	}	
}
end
}

******************************************************
* create global with time horizon
** if set is missing, replace with default
if "$prismod_end" == "" {
	global end_horizon = tq(2030q4)
}
** if present, use it
else {
	global end_horizon = $prismod_end
}

******************************************************

	global entrylag = 4
	scalar fr_eligible=0.25	

*** 1. CREATE TIMELAGS IN MODEL BY MOVING COHORTS IN TIME
*** c1 is lagged according to time
quietly {
capture program drop pminflowsimul
program define pminflowsimul	
	args p
	quietly {
	noisily display "Shifting cohorts in time... (step 1 of 8)" as result		
	* use future scenario parameters
	use simulations_$pcent/`p'/`p'_paramavg_$pcent
	
* 1. Lag between start of tr. rizeni a start of tr. stihani
	g midpoint = q + 0.5
	g mid_lagged=midpoint+z1		
	* by construction, cohort can only be distributed between two quarters
	* l - lower bound of interval  and u - upper bound of interval  
	g l_bound=mid_lagged-0.5
	g u_bound=mid_lagged+0.5		
	* calculate percentage and number of cases present in lower or upper quarter
	g l_prec=(ceil(l_bound)-l_bound)*c1
	** case when l_charged is integer, then all cases are in lower bound quarter
	replace l_prec=1*c1 if round(l_bound)==l_bound
	** upper bound
	g u_prec=(u_bound-int(u_bound))*c1			
	* generate quarters for each bound
	g l_q=int(l_bound)
	g u_q=int(u_bound)		
	* save as separate dataset for merging
	sort oca q
	save temp.dta, replace		
	* keep only values for lower quarter
	keep l_q l_prec oca
	* collapse as sum according to lower bound quarter
	collapse (sum) l_prec, by(l_q oca)
	rename l_prec c1_low
	rename l_q q
	sort oca q
	save temp1.dta, replace		
	* keep only values for higher quarter
	use  temp.dta
	keep u_q u_prec oca
	* collapse as sum according to upper bound quarter
	collapse (sum) u_prec, by(u_q oca)
	rename u_prec c1_up
	rename u_q q
	sort oca q
	save temp2.dta, replace		
	* merge tempfiles together
	use temp.dta
	merge oca q using temp1.dta
	replace c1_low=0 if missing(c1_low)
	drop _merge
	sort oca q
	merge oca q using temp2.dta
	replace c1_up=0 if missing(c1_up)
	drop _merge
	sort oca q		
	* sum c1 and erase tempfiles
	g c1_policeend=c1_up+c1_low
	label var c1_policeend "number tr. rizeni and the end of z1"
	drop c1_up c1_low mid_lagged l_bound u_bound l_prec u_prec  l_q u_q midpoint
	erase temp.dta
	erase temp1.dta
	erase temp2.dta
	* number of tr. stihani started by p1 and corrected by p12
	g c1_toprosecuted=c1_policeend*p1*p12
	label var c1_toprosecuted "number of cases from police to prosecutors"	

* 2. Lag between start and end of prosecution	
	* define midpoint variable for quarter
	g midpoint = q + 0.5
	g mid_lagged=midpoint+z2		
	* by construction, cohort can only be distributed between two quarters
	* l - lower bound of interval  and u - upper bound of interval  
	g l_bound=mid_lagged-0.5
	g u_bound=mid_lagged+0.5		
	* calculate percentage and number of cases present in lower or upper quarter
	g l_prec=(ceil(l_bound)-l_bound)*c1_toprosecuted
	** case when l_charged is integer, then all cases are in lower bound quarter
	replace l_prec=1*c1_toprosecuted if round(l_bound)==l_bound
	** upper bound
	g u_prec=(u_bound-int(u_bound))*c1_toprosecuted			
	* generate quarters for each bound
	g l_q=int(l_bound)
	g u_q=int(u_bound)		
	* save as separate dataset for merging
	sort oca q
	save temp.dta, replace		
	* keep only values for lower quarter
	keep l_q l_prec oca
	* collapse as sum according to lower bound quarter
	collapse (sum) l_prec, by(l_q oca)
	rename l_prec c2_low
	rename l_q q
	sort oca q
	save temp1.dta, replace		
	* keep only values for higher quarter
	use  temp.dta
	keep u_q u_prec oca
	* collapse as sum according to upper bound quarter
	collapse (sum) u_prec, by(u_q oca)
	rename u_prec c2_up
	rename u_q q
	sort oca q
	save temp2.dta, replace		
	* merge tempfiles together
	use temp.dta
	merge oca q using temp1.dta
	replace c2_low=0 if missing(c2_low)
	drop _merge
	sort oca q
	merge oca q using temp2.dta
	replace c2_up=0 if missing(c2_up)
	drop _merge
	sort oca q		
	* sum c2 and erase tempfiles
	g c2_prosecuted=c2_up+c2_low
	drop c2_up c2_low mid_lagged l_bound u_bound l_prec u_prec  l_q u_q midpoint
	erase temp.dta
	erase temp1.dta
	erase temp2.dta
	label var c2_prosecuted "number of cases prosecuted, at time of charge"
	* from c2 prosecuted only c2_charged*p2 is actually charged and continues to courts
	*! *p23 - fraction of cases to 
	g c2_charged=c2_prosecuted*p2*p23
	label var c2_charged "number of cases charged, at time of charge"
	
* 1.2. Lag between charge and conviction
	*** input: z3, c2_charged; output: c2_convinced
	* define midpoint variable for quarter
	g midpoint = q + 0.5
	g mid_lagged=midpoint+z3		
	* by construction, cohort can only be distributed between two quarters
	*l - lower bound of interval  and u - upper bound of interval  
	g l_bound=mid_lagged-0.5
	g u_bound=mid_lagged+0.5		
	* calculate percentage and number of cases present in lower or upper quarter
	g l_prec=(ceil(l_bound)-l_bound)*c2_charged
	** case when l_charged is integer, then all cases are in lower bound quarter
	replace l_prec=1*c2_charged if round(l_bound)==l_bound
	** upper bound
	g u_prec=(u_bound-int(u_bound))*c2_charged			
	* generate quarters for each bound
	g l_q=int(l_bound)
	g u_q=int(u_bound)		
	* save as separate dataset for merging
	sort oca q
	save temp.dta, replace		
	* keep only values for lower quarter
	keep l_q l_prec oca
	* collapse as sum according to lower bound quarter
	collapse (sum) l_prec, by(l_q oca)
	rename l_prec c2_low
	rename l_q q
	sort oca q
	save temp1.dta, replace		
	* keep only values for higher quarter
	use  temp.dta
	keep u_q u_prec oca
	* collapse as sum according to upper bound quarter
	collapse (sum) u_prec, by(u_q oca)
	rename u_prec c2_up
	rename u_q q
	sort oca q
	save temp2.dta, replace		
	* merge tempfiles together
	use temp.dta
	merge oca q using temp1.dta
	replace c2_low=0 if missing(c2_low)
	drop _merge
	sort oca q
	merge oca q using temp2.dta
	replace c2_up=0 if missing(c2_up)
	drop _merge
	sort oca q		
	* sum c2 and erase tempfiles
	g c2_trial=c2_up+c2_low
	drop c2_up c2_low mid_lagged l_bound u_bound l_prec u_prec  l_q u_q midpoint
	erase temp.dta
	erase temp1.dta
	erase temp2.dta
	label var c2_trial "number of cases charged, at time of trial end"	
	* from c2_trial only c2_trial*p3 are convicted at the end and from them only  c2_conviced*p4 goes to prison and c2_conviced*(p_a1+p_a2) gets alternative sentences
	g c2_conviced=c2_trial*p3
	g c2_toprison= c2_conviced*p4
	g c2_alternative=c2_conviced*(p_a1+p_a2)
	label var c2_conviced "number of cases convicted, at time of trial end"
	label var c2_toprison "number of cases sentenced to prison, at time of trial end"
	label var c2_alternative "number of cases sentenced to alt. punish, at time of trial end"
	
* 3. Entry to prison
	** 3.1 Alternative sentences
	* alternative entrants enter population at risk right after they are sentenced
	g nepota1 = c2_conviced*p_a1
	g nepota2 = c2_conviced*p_a2
	label var nepota1 "Inflow to stock at risk of suspended sentences"
	label var nepota2 "Inflow to stock at risk community service sentences"
			
	** 3.2 Normal sentences
	*** corrected by p5 and p45 to include only entered sentences
	g inflowr_rp=c2_conviced*p4*(1-p5)*p45	
	label var inflowr_rp "Inflow of regular prisoners"	
	
noisily display "Done!" as result	
	
*** 2. GENERATE THE STOCK OF POTENTIAL PAROLE / COMMUNITY SERVECE VIOLATIONS	
	
	scalar corr_hs= 0.65	
	scalar gamma = 0.064 
	scalar corr_ls= (1-gamma*corr_hs)/(1-gamma)
	
	** 1. Simulate stock at risk
	noisily display "Simulating population at risk for parole... (step 2 of 8)" as result
	simul_stockatrisk `p' nepota1 vr_a1 la1 amr_a1 a1
	noisily display "Done!" as result

	noisily display "Simulating population at risk for public service.. (step 3 of 8)" as result
	simul_stockatrisk `p' nepota2 vr_a2 la2 amr_a2 a2
	noisily display "Done!" as result

	** label variables
	label var potastock_a1 "stock of potential parole violators by oca"
	label var potainflow_a1 "inflow of potential parole violators by oca"
	label var potaoutflow_a1 "outflow of potential parole violators by oca"
	label var totpotastock_a1 "total stock of potential parole violators"
	label var totpotainflow_a1 "total inflow of potential parole violators"
	label var totpotaoutflow_a1  "total outflow of potential parole violators"
	label var potastock_a2 "stock of potential pub. service violators by oca"
	label var potainflow_a2 "inflow of potential pub. service violators by oca"
	label var potaoutflow_a2 "outflow of potential pub. service violators by oca"
	label var totpotastock_a2 "total stock of potential pub. service violators"
	label var totpotainflow_a2 "total inflow of potential pub. service violators"
	label var totpotaoutflow_a2  "total outflow of potential pub. service violators"


*** 3. SIMULATE STOCK/FLOWS FROM DATASET
noisily display "Simulating stock of regular prisoners... (step 4 of 8)" as result	

** save sentences
g save5=s_ls
g save6=s_hs
g save7=s_a1
g save8=s_a2
g save9=s_a3
** 1. Regular prisoners
*** 1.1. Simulate 1st part 
	** reduce inflow by fr entry
	g save1=inflowr_rp
	replace inflowr_rp=inflowr_rp*p_z4
	** simulate stock
	simul_prisonstock `p' inflowr_rp s_ls pe_a3 pe_o lc amv_rp rp
	** rename generated variables and tempfiles
	cap drop outflow_p7_* 
	cap drop outflow_p6_*
	cap drop outflow_exp*
	foreach end in rpls hs lshs { 
		foreach var of varlist stock_`end' inflow_`end' outflow_`end' totstock_`end' totinflow_`end' totoutflow_`end'  {
			cap rename `var' `var'1
		}
		save temp.dta, replace		
		use temp.dta
	}
noisily display "..... 50% done with simulating stock of regular prisoners!" as result	
** 1.2 Generate 2nd part
	replace inflowr_rp=save1*(1-p_z4)
	replace save1=0 if missing(save1)
	* lag inflow, lag sentences by Xq
	** lag is defined on top of the file
	* easy solution because lag is time independent
	** need to use temp varibles
		g temp_1=0
		g temp_2=0
		g temp_3=0
	forval i = 1/9 {
		replace temp_1=L$entrylag.inflowr_rp if oca==`i' 
		replace inflowr_rp=temp_1 if oca==`i' 
		replace temp_2=L$entrylag.s_ls  if oca==`i'  
		replace s_ls=temp_2 if oca==`i'  
		replace temp_3=L$entrylag.s_hs if oca==`i' 
		replace s_hs=temp_3 if oca==`i' 
		replace inflowr_rp=0 if missing(inflowr_rp)
		replace s_ls=0 if missing(s_ls)
		replace s_hs=0 if missing(s_hs)
	}
	drop temp_*
	* simulate stock
	simul_prisonstock `p' inflowr_rp s_ls pe_a3 pe_o lc amv_rp rp
	* rename generated variables and generated files
	cap drop outflow_p7_* 
	cap drop outflow_p6_*
	cap drop outflow_exp*
	foreach end in rpls hs lshs { 
		foreach var of varlist stock_`end' inflow_`end' outflow_`end' totstock_`end' totinflow_`end' totoutflow_`end' {
			cap rename `var' `var'2
		}
		save temp.dta, replace		
		use temp.dta
	}
noisily display "Done!" as result
		
** 2. Alternatiave sentences
noisily display "Simulating stock of parole and public service violators... (step 5 of 8) " as result	
*** 2.1. Simulate 1st part
	g save2=inflowr_a1
	g save3=inflowr_a2
	replace inflowr_a1=inflowr_a1*p_z4
	replace inflowr_a2=inflowr_a2*p_z4
	simul_prisonstock `p' inflowr_a1 s_a1 pe_a3 pe_o lc amv_a1 a1
	simul_prisonstock `p' inflowr_a2 s_a2 pe_a3 pe_o lc amv_a2 a2
	* rename vars
	cap drop outflow_p7_* 
	cap drop outflow_p6_*
	cap drop outflow_exp*
	foreach end in a1 a2 { 
		foreach var of varlist stock_`end' inflow_`end' outflow_`end' totstock_`end' totinflow_`end' totoutflow_`end' {
			cap rename `var' `var'1
			}
		save temp.dta, replace		
		use temp.dta	
	}		
*** 2.2. Simulate 2nd part
	replace inflowr_a1=save2*(1-p_z4)
	replace inflowr_a2=save3*(1-p_z4)
	replace save2=0 if missing(save1)
	replace save3=0 if missing(save1)
	* lag inflow, lag sentences by 3q
	* easy solution of lag is time independent
	** need to use temp varibles	
	forval i = 1/9 {
		g temp=0
		foreach var of varlist inflowr_a1 inflowr_a2 s_a1 s_a2 {
			replace temp=L$entrylag.`var' if  oca==`i' 
			replace `var'=temp if  oca==`i' 
			replace `var'=`var' if missing(`var')
		}
		drop temp*
	}	
	* simulate stocks
	simul_prisonstock `p' inflowr_a1 s_a1 pe_a3 pe_o lc amv_a1 a1
	simul_prisonstock `p' inflowr_a2 s_a2 pe_a3 pe_o lc amv_a2 a2

	* rename vars
	cap drop outflow_p7_* 
	cap drop outflow_p6_*
	cap drop outflow_exp*
	foreach end in a1 a2 { 
		foreach var of varlist stock_`end' inflow_`end' outflow_`end' totstock_`end' totinflow_`end' totoutflow_`end' {
			cap rename `var' `var'2
			}
		save temp.dta, replace		
		use temp.dta	
	}	
noisily display "Done!" as result
	
*** 3. Generate paroled prisoner population and stock
** 3.1 Calculate number of entrants to stock at risk and simulate population at risk
	egen tempstock = rowtotal(stock_rpls1 stock_hs1 stock_rpls2 stock_hs2 stock_a11 stock_a22 stock_a12 stock_a22 stock_lshs1 stock_lshs2)
	** stock of prisoner*prop. of prison parole
	g nepota3=tempstock*pe_a3
	label var nepota3 "Inflow from prison parole to stock at risk"
	drop tempstock
	
	noisily display "Simulating population at risk of prison parole... (step 6 of 8)" as result	
	simul_stockatrisk `p' nepota3 vr_a3 la3 amr_a3 a3
	noisily display "Done!" as result

	label var potastock_a3 "stock of potential prison parole violators by oca"
	label var potainflow_a3 "inflow of potential prison parole violators by oca"
	label var potaoutflow_a3 "outflow of potential prison parole violators by oca"
	label var totpotastock_a3 "total stock of potential prison parole violators"
	label var totpotainflow_a3 "total inflow of potential prison parole violators"
	label var totpotaoutflow_a3  "total outflow of potential prison parole violators"

	
** 3.2 Prison parole prisoners stocks
noisily display "Simulating stock of prison parole violators... (step 7 of 8)" as result	
*** 1st part of the stock
	g save4=inflowr_a3
	replace inflowr_a3=inflowr_a3*p_z4
	simul_prisonstock `p' inflowr_a3 s_a3 pe_a3 pe_o lc amv_a3 a3
	cap drop outflow_p7_* 
	cap drop outflow_p6_*
	cap drop outflow_exp*
	foreach end in a3 { 
		foreach var of varlist stock_`end' inflow_`end' outflow_`end' totstock_`end' totinflow_`end' totoutflow_`end' {
			cap rename `var' `var'1
			}
		save temp.dta, replace		
		use temp.dta	
	}		
*** 2nd part of the stock
	replace save4=0 if missing(save4)
	replace inflowr_a3=save4*(1-p_z4)
	forval i = 1/9 {
		g temp=0
		foreach var of varlist inflowr_a3 s_a3 {
			replace temp=L$entrylag.`var' if  oca==`i' 
			replace `var'=temp if  oca==`i' 
			replace `var'=`var' if missing(`var')
		}
		drop temp*
	}	
	simul_prisonstock `p' inflowr_a3 s_a3 pe_a3 pe_o lc amv_a3 a3
	cap drop outflow_p7_* 
	cap drop outflow_p6_*
	cap drop outflow_exp*
	foreach end in a3 { 
		foreach var of varlist stock_`end' inflow_`end' outflow_`end' totstock_`end' totinflow_`end' totoutflow_`end' {
			cap rename `var' `var'2
			}
		save temp.dta, replace		
		use temp.dta	
	}	
noisily display "Done!" as result

*** 4. Generate  summary variables and finish
noisily display "Generating summary variables... (step 8 of 8)" as result
sort oca q
** remerge stock and outflows from different parts
foreach end in rpls hs lshs a1 a2 a3 {
	g stock_`end'=stock_`end'1+stock_`end'2
	g totstock_`end'=totstock_`end'1+totstock_`end'2
	g outflow_`end'=outflow_`end'1+outflow_`end'2
	g totoutflow_`end'=totoutflow_`end'1+totoutflow_`end'2
	g inflow_`end'=inflow_`end'1+inflow_`end'2
	g totinflow_`end'=totinflow_`end'1+totinflow_`end'2
}

** restore variables
replace inflowr_rp=save1
replace inflowr_a1=save2
replace inflowr_a2=save3
replace inflowr_a3=save4
replace s_ls=save5
replace s_hs=save6
replace s_a1=save7
replace s_a2=save8
replace s_a3=save9
drop save*

** generate summary stocks
g stock = stock_rpls + stock_hs + stock_lshs+stock_a1 + stock_a2 + stock_a3
g stock_rp=stock_rpls+stock_hs+stock_lshs
g stock_ls=stock_rpls+stock_a1+stock_a2+stock_a3+stock_lshs
egen totstock = total(stock), by(q)
egen totstock_rp = total(stock_rp), by(q)
egen totstock_ls = total(stock_ls), by(q)
g inflow=inflow_a1 + inflow_a2 + inflow_a3 + inflow_rpls + inflow_hs + inflow_lshs
g outflow=outflow_a1+outflow_a2 + outflow_a3 + outflow_rpls + outflow_lshs+ outflow_hs
g totinflow=totinflow_rpls + totinflow_a1 + totinflow_a2+ totinflow_a3 + totinflow_hs +totinflow_lshs
g totoutflow=totoutflow_rpls+totoutflow_a1+totoutflow_a2+ totoutflow_a3 + totoutflow_hs + totoutflow_lshs

g inflow_rp=inflow_rpls+inflow_hs+inflow_lshs
g outflow_rp=outflow_rpls+outflow_hs+outflow_lshs
g inflow_ls=inflow_rpls+inflow_a1+inflow_a2+inflow_a3+inflow_lshs
g outflow_ls=outflow_rpls+outflow_a1+outflow_a2+outflow_a3+outflow_lshs

egen totinflow_rp = total(inflow_rp), by(q)
egen totoutflow_rp = total(outflow_rp), by(q)
egen totinflow_ls = total(inflow_ls), by(q)
egen totoutflow_ls = total(outflow_ls), by(q)


label var stock "stock of all prisoners, by oca"
label var totstock "total stock of all prisoners"
label var inflow "simulated inflow of  prisoners, by oca"
label var outflow "outflow of  prisoners, by oca"
label var totinflow "total inflow of all prisoners"
label var totoutflow  "total outflow of all prisoners"

cap drop id

label var inflowr_a1 "inflow of a1 to prisons"
label var inflowr_a2 "inflow of a2 to prisons"
label var inflowr_a3 "inflow of a3 to prisons"

foreach suffix in rpls hs lshs a1 a2 a3 rp ls {
	label var stock_`suffix' `"stock of `suffix' prisoners, by oca"'
	label var totstock_`suffix' `"total stock of `suffix' prisoners"'
	label var outflow_`suffix' `"outflow of `suffix' prisoners, by oca"'
	label var totoutflow_`suffix' `"total outflow of `suffix' prisoners"'
	label var inflow_`suffix' `"inflow of `suffix' prisoners, by oca"'
	label var totinflow_`suffix' `"total inflow of `suffix' prisoners"'
}

* drop partial stock, leave only totals
drop stock_hs1 inflow_hs1 outflow_hs1 totstock_hs1 totinflow_hs1 totoutflow_hs1 stock_lshs1 inflow_lshs1 outflow_lshs1 totstock_lshs1 totinflow_lshs1 totoutflow_lshs1 stock_rpls1 inflow_rpls1 outflow_rpls1 totstock_rpls1 totinflow_rpls1 totoutflow_rpls1 stock_hs2 inflow_hs2 outflow_hs2 totstock_hs2 totinflow_hs2 totoutflow_hs2 stock_lshs2 inflow_lshs2 outflow_lshs2 totstock_lshs2 totinflow_lshs2 totoutflow_lshs2 stock_rpls2 inflow_rpls2 outflow_rpls2 totstock_rpls2 totinflow_rpls2 totoutflow_rpls2 stock_a11 inflow_a11 outflow_a11 totstock_a11 totinflow_a11 totoutflow_a11 stock_a21 inflow_a21 outflow_a21 totstock_a21 totinflow_a21 totoutflow_a21 stock_a12 inflow_a12 outflow_a12 totstock_a12 totinflow_a12 totoutflow_a12 stock_a22 inflow_a22 outflow_a22 totstock_a22 totinflow_a22 totoutflow_a22
drop  stock_a31 inflow_a31 outflow_a31 totstock_a31 totinflow_a31 totoutflow_a31 stock_a32 inflow_a32 outflow_a32 totstock_a32 totinflow_a32 totoutflow_a32
sort oca q
save simulations_$pcent/`p'/`p'_wmodelavg_$pcent, replace

*end quietly
}
* drop used globals
macro drop prismod_end
macro drop end_horizon
cap erase temp.dta

noisily display "Finished!" as result
end
}


